@@ -1,10 +1,10 @@
 //
-// $Id: sphinxexcerpt.cpp 3022 2011-11-21 15:58:19Z tomat $
+// $Id: sphinxexcerpt.cpp 3087 2012-01-30 23:07:35Z shodan $
 //
 
 //
-// Copyright (c) 2001-2011, Andrew Aksyonoff
-// Copyright (c) 2008-2011, Sphinx Technologies Inc
+// Copyright (c) 2001-2012, Andrew Aksyonoff
+// Copyright (c) 2008-2012, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -313,7 +313,7 @@ struct SnippetsQword_Exact_c: public ISnippetsQword
 		while ( m_iToken < m_dTokens->GetLength() )
 		{
 			Token_t & tToken = (*m_dTokens)[m_iToken++];
-			if ( tToken.m_eType!=ExcerptGen_c::TOK_WORD )
+			if ( !( tToken.m_eType==ExcerptGen_c::TOK_WORD || tToken.m_eType==ExcerptGen_c::TOK_SPZ ) )
 				continue;
 
 			if ( tToken.m_iWordID==m_iWordID || tToken.m_iBlendID==m_iWordID )
@@ -739,9 +739,17 @@ void ExcerptGen_c::TokenizeDocument ( char * pData, int iDataLen, CSphDict * pDi
 
 			if ( iSPZ && *sWord>=iSPZ && ( m_dTokens.GetLength()==0 || m_dTokens.Last().m_eType!=TOK_SPZ ) )
 			{
+				BYTE * sWordSPZ = sWord;
+				if ( (*sWord)==MAGIC_CODE_SENTENCE )
+					sWordSPZ = (BYTE *)MAGIC_WORD_SENTENCE;
+				else if ( (*sWord)==MAGIC_CODE_PARAGRAPH )
+					sWordSPZ = (BYTE *)MAGIC_WORD_PARAGRAPH;
+
 				Token_t & tLast = m_dTokens.Add();
 				tLast.Reset();
 				tLast.m_eType = TOK_SPZ;
+				tLast.m_iWordID = pDict->GetWordID ( sWordSPZ );
+				tLast.m_uPosition = uPosition;
 
 				if ( *sWord==MAGIC_CODE_SENTENCE )
 				{
@@ -1894,6 +1902,7 @@ public:
 	int			FindWord ( SphWordID_t iWordID, const BYTE * sWord, int iWordLen ) const;
 	void		AddHits ( SphWordID_t iWordID, const BYTE * sWord, int iWordLen, DWORD uPosition );
 	bool		Parse ( const char * sQuery, ISphTokenizer * pTokenizer, CSphDict * pDict, const CSphSchema * pSchema, CSphString & sError, int iStopwordStep );
+	int			GetSPZ () const;
 
 protected:
 	bool		MatchStar ( const ExcerptGen_c::Keyword_t & tTok, const BYTE * sWord, int iWordLen ) const;
@@ -2039,6 +2048,24 @@ bool SnippetsDocIndex_c::Parse ( const char * sQuery, ISphTokenizer * pTokenizer
 }
 
 
+int SnippetsDocIndex_c::GetSPZ () const
+{
+	// with sentence in query we should consider SENTECE, PARAGRAPH, ZONE
+	// with paragraph in query we should consider PARAGRAPH, ZONE
+	// with zone in query we should consider ZONE
+	if ( m_bSentence )
+		return MAGIC_CODE_SENTENCE;
+
+	if ( m_bParagraph )
+		return MAGIC_CODE_PARAGRAPH;
+
+	if ( m_tQuery.m_dZones.GetLength() )
+		return MAGIC_CODE_ZONE;
+
+	return 0;
+}
+
+
 void SnippetsDocIndex_c::AddWord ( SphWordID_t iWordID )
 {
 	assert ( iWordID );
@@ -2776,6 +2803,10 @@ static char * HighlightAllFastpath ( const ExcerptQuery_t & tQuerySettings,
 	if ( !tContainer.Parse ( tFixedSettings.m_sWords.cstr(), pQueryTokenizer, pDict, pSchema, sError, tIndexSettings.m_iStopwordStep ) )
 		return NULL;
 
+	// fast-path collects no passages but that flag says what SPZ should we collect
+	if ( tFixedSettings.m_bHighlightQuery && !tFixedSettings.m_iPassageBoundary )
+		tFixedSettings.m_iPassageBoundary = tContainer.GetSPZ();
+
 	// do highlighting
 	if ( !tFixedSettings.m_bHighlightQuery )
 	{
@@ -3041,5 +3072,5 @@ char * sphBuildExcerpt ( ExcerptQuery_t & tOptions, CSphDict * pDict, ISphTokeni
 }
 
 //
-// $Id: sphinxexcerpt.cpp 3022 2011-11-21 15:58:19Z tomat $
+// $Id: sphinxexcerpt.cpp 3087 2012-01-30 23:07:35Z shodan $
 //
