@@ -1,10 +1,10 @@
 //
-// $Id: sphinxstd.cpp 3008 2011-11-13 12:36:32Z klirichek $
+// $Id: sphinxstd.cpp 3130 2012-03-01 07:43:56Z tomat $
 //
 
 //
-// Copyright (c) 2001-2011, Andrew Aksyonoff
-// Copyright (c) 2008-2011, Sphinx Technologies Inc
+// Copyright (c) 2001-2012, Andrew Aksyonoff
+// Copyright (c) 2008-2012, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -1043,6 +1043,21 @@ void * sphMyStack ()
 	return sphThreadGet ( g_tMyThreadStack );
 }
 
+
+int64_t sphGetStackUsed()
+{
+	BYTE cStack;
+	BYTE * pStackTop = (BYTE*)sphMyStack();
+	if ( !pStackTop )
+		return 0;
+	int64_t iHeight = pStackTop - &cStack;
+	if ( iHeight>=0 )
+		return iHeight;
+	else
+		return -iHeight;
+}
+
+
 int sphMyStackSize ()
 {
 #if USE_WINDOWS
@@ -1304,5 +1319,5 @@ bool CSphRwlock::Unlock ()
 #endif
 
 //
-// $Id: sphinxstd.cpp 3008 2011-11-13 12:36:32Z klirichek $
+// $Id: sphinxstd.cpp 3130 2012-03-01 07:43:56Z tomat $
 //
